from pathlib import Path

from docx import Document


class Saver(object):
    def __init__(self, song: str, artist: str, translation: str, language: str):
        self.song = song
        self.artist = artist
        self.translation = translation
        self.language = language

    def _get_header(self) -> str:
        """_summary_

        Returns:
            str: _description_
        """
        return f"{self.song} - {self.artist} in {self.language}"

    def _get_name(self) -> str:
        """_summary_

        Returns:
            str: _description_
        """
        return f"{self.language}__{self.song}__{self.artist}".replace(" ", "_").lower()

    def to_word(self, folder: Path):
        document = Document()
        document.add_heading(self.header, level=1)
        document.add_paragraph(self.translation)
        document.save(Path(folder / f"{self.name}.docx"))

    def to_txt(self, folder: Path):
        with open(Path(folder / f"{self.name}.txt"), mode="w", encoding="utf-8") as f:
            f.write(self.translation)

    def save(self, folder: Path, kind: str = "txt") -> None:
        """_summary_

        Args:
            folder (Path): _description_
            kind (str, optional): _description_. Defaults to "txt".
        """
        self.header = self._get_header()
        self.name = self._get_name()
        if kind == "word":
            self.to_word(folder)
        elif kind == "txt":
            self.to_txt(folder)



from pathlib import Path
from typing import Optional

from lyrics_translator.core.saver import Saver
from lyrics_translator.core.utils import MockGeniusSong


class Lyrics(object):
    def __init__(self, song: str, artist: str, testing: bool = None) -> None:
        self.song = song
        self.artist = artist
        self.testing = testing

        self.text: str = None
        self.translation: str = None
        self.language: str = None

    def download_lyrics(self, genius, get_full_info: Optional[bool] = False) -> None:
        """Download the lyrics of the song using the `genius` API.

        Args:
            get_full_info (bool): _description_. Defaults to False.

        Returns:
            str: _description_
        """
        if self.testing:
            genius_song = MockGeniusSong(lyrics="<test lyrics>")
        else:
            genius_song = genius.search_song(
                self.song, self.artist, get_full_info=get_full_info
            )

        if not genius_song:
            message = f"not found - song: {self.song}, artist: {self.artist}"
            raise ValueError(message)

        self.text = genius_song.lyrics

    def translate(self, translator, language: str) -> None:
        """Translate the lyrics into the target language.

        Returns:
            str: _description_
        """
        self.language = language

        if self.testing:
            self.translation = "<test translation>"
        else:
            self.translation = translator.translate(self.text)

    def get_translation(self, translator, language) -> str:
        self.translate(translator, language)
        return self.translation

    def get_lyrics(self, genius) -> str:
        self.download_lyrics(genius)
        return self.text

    def save(self, folder: str = "lyrics", kind: str = "txt") -> None:
        """Save the translated lyrics to a file.

        Args:
            folder (Path): _description_
            kind (str, optional): _description_. Defaults to "txt".
        """

        if not folder:
            folder_path = Path().resolve()
        else:
            folder_path = Path(folder).resolve()
            folder_path.mkdir(parents=True, exist_ok=True)

        saver = Saver(
            song=self.song,
            artist=self.artist,
            translation=self.translation,
            language=self.language,
        )
        saver.save(folder=folder_path, kind=kind)

    def __str__(self) -> str:
        """string version of the Lyrics instance, returns the translated lyrics

        Returns:
            str: _description_
        """
        return self.translation

    def __repr__(self) -> str:
        """Lyrics representation

        Returns:
            str: returns the string to create the same instance
        """
        return f"Lyrics(song={self.song}, artist={self.artist})"



import json
from pathlib import Path


def split_lyrics(text):
    return text.replace("[", "\n----\n[").split("\n")


def get_base_path():
    return Path(__file__).parent.parent.absolute()


def create_folder(path):
    """
    create folder, if it doesn't already exist
    """
    Path(path).mkdir(parents=True, exist_ok=True)
    return path


def read_json(file_path):
    with open(file_path, "r") as f:
        return json.load(f)


def read_songs(folder: Path, file: str = "songs.txt"):
    with open(Path(folder / file), encoding="utf-8") as f:
        songs = f.read()
    return [song.split(",") for song in songs.split("\n")]


class MockGeniusSong:
    def __init__(self, lyrics):
        self.lyrics = lyrics



from functools import lru_cache

import torch
from transformers import AutoModelForSeq2SeqLM, AutoTokenizer, pipeline

torch.multiprocessing.freeze_support()


cache = {}


class Translator(object):
    def __init__(self, language: str, origin_language: str = "en"):
        self.language = language
        self.origin_language = origin_language

    @lru_cache(maxsize=5)
    def get_translator_pipeline(self) -> None:
        """[summary]

        Raises:
            ValueError: [description]

        """
        if self.language == "de" and self.origin_language == "en":
            model_name = "t5-small"
        else:
            print("trying to find the model....")
            model_name = f"Helsinki-NLP/opus-mt-{self.origin_language}-{self.language}"

        if model_name in cache:
            self.translator = cache[model_name]
        else:
            try:
                tokenizer = AutoTokenizer.from_pretrained(
                    model_name, model_max_length=512, skip_special_tokens=True
                )
                model = AutoModelForSeq2SeqLM.from_pretrained(model_name)

            except OSError as err:
                message = f'{err} -- language "{self.language}" is not supported!'
                raise ValueError(message)

            self.translator = pipeline(
                f"translation_{self.origin_language}_to_{self.language}",
                model=model,
                tokenizer=tokenizer,
            )
            cache[model_name] = self.translator

        print(
            f'Setup translator_pipeline to translate from "{self.origin_language}" to'
            f' "{self.language}" using "{model_name}"!'
        )

    def translate(self, text: str) -> str:
        """[summary]

        Args:
            text (str): [description]

        Returns:
            str: [description]
        """
        print("Song is being translated, this can take a while...")

        skip_token = "_11111111_"

        sentences = [
            sentence.strip() if sentence.strip() != "" else skip_token
            for sentence in text.strip().split("\n")
        ]
        output = []
        for translation in self.translator(sentences, batch_size=8):
            output.append(translation["translation_text"])

        output_sentences = "\n".join(output)
        return output_sentences.replace(skip_token, "\n")

    def load_config():
        """to be implemented"""
        pass


if __name__ == "__main__":

    language = "de"  # "de"
    short_text = "Hello my friends! How are you doing today?"

    long_text = """
        Importing AutoTokenizer and AutoModelForSeq2SeqLM from transformers. Note that you need to import TFAutoModelForSeq2SeqLM if you want the TensorFlow equivalent.
        Initializing the Tokenizer. We’ll be using the Helsinki-NLP pretrained/finetuned OpusMT English to Dutch model for initializing the tokenizer. Using a tokenizer, we can convert textual inputs into tokens.
        Initializing the model. Using the same pretrained/finetuned model, we can generate translations.
        We then tokenize the input text in a Seq2Seq fashion as if we convert a batch of one sentence (hence wrapping everything inside a Python list).
        We then generate a translation for all the elements in the batch, decode the batch, and take the first element.
        Which is the translated text that we then print on screen.
    """

    lyrics = """
        Surfin’ USA Lyrics[Verse 1]
        If everybody had an ocean
        Across the U.S.A
        Then everybody'd be surfin'
        Like Californi-a
        You'd see them wearing their baggies
        Huarache sandals too
        A bushy bushy blond hairdo
        Surfin' U.S.A

        [Chorus]
        You'd catch 'em surfin' at Del Mar
        (Inside, outside, U.S.A.)
        Ventura County line
        (Inside, outside, U.S.A.)
        Santa Cruz and Trestles
        (Inside, outside, U.S.A.)
        Australia's Narrabeen
        (Inside, outside, U.S.A.)
        All over Manhattan
        (Inside, outside, U.S.A.)
        And down Doheny Way
        (Inside, outside)
        [Hook]
        Everybody's gone surfin'
        Surfin' U.S.A

        [Verse 2]
        We'll all be planning that route
        We're gonna take real soon
        We're waxing down our surfboards
        We can't wait for June
        We'll all be gone for the summer
        We're on surfari to stay
        Tell the teacher we're surfin'
        Surfin' U.S.A

        [Chorus]
        Haggerties and Swamis
        (Inside, outside, U.S.A.)
        Pacific Palisades
        (Inside, outside, U.S.A.)
        San Onofre and Sunset
        (Inside, outside, U.S.A.)
        Redondo Beach LA
        (Inside, outside, U.S.A.)
        All over La Jolla
        (Inside, outside, U.S.A.)
        At Wa'imea Bay
        (Inside, outside)
        You might also like[Hook]
        Everybody's gone surfin'
        Surfin' U.S.A

        [Instrumental Interlude]

        [Outro]
        Everybody's gone surfin'
        Surfin' U.S.A

        Everybody's gone surfin'
        Surfin' U.S.A

        Yeah everybody's gone surfin'
        Surfin' U.S.A

        Yeah everybody's gone surfin'
        Surfin' U.S.A12Embed
        """
    translator = Translator(language)
    translator.get_translator_pipeline()
    # translation = translator.translate(long_text)
    translation = translator.translate_fast(lyrics)

    print("----" * 10)
    print(translation)



import lyricsgenius
from dotenv import dotenv_values

from lyrics_translator.core.lyrics import Lyrics
from lyrics_translator.core.translator import Translator

MANDATORY_ENV_VARS = ["GENIUS_ACCESS_TOKEN"]


class LyricsTranslator(object):
    def __init__(
        self, language: str = "de", origin_language="en", config: dict = None
    ) -> None:
        """LyricsTranslator main class, which uses the Lyrics class to fetch
        lyrics and translates them into the target language.

        Args:
            language (str): target language that the lyrics should be transalted into. Defaults to "de".
            origin_language (str, optional): set optional current language of the lyrics for more advanced traslations. Defaults to "en".
            config (dict): config file to pass in environment variables

        Raises:
            EnvironmentError: _description_
        """
        self.language = language
        self.origin_language = origin_language

        if config is None:
            self.config = dotenv_values(".env")
        else:
            self.config = config

        for env_var in MANDATORY_ENV_VARS:
            if env_var not in self.config:
                message = (
                    f"Failed because the envrionment variable '{env_var}' is not set."
                    f" Add '{env_var}' to the '.env' file and try it again!"
                )
                raise EnvironmentError(message)

        self.translator = Translator(
            language=self.language, origin_language=self.origin_language
        )
        self.translator.get_translator_pipeline()

        self.genius = lyricsgenius.Genius(
            self.config["GENIUS_ACCESS_TOKEN"], timeout=10, retries=3
        )

    def get_song_lyrics(self, song, artist, testing: bool = False):
        """retrun only the lyrics of the song"""
        lyrics = Lyrics(song=song, artist=artist, testing=testing)
        text = lyrics.get_lyrics(genius=self.genius)
        return lyrics

    def get_song_translation(self, song, artist, testing: bool = False) -> None:
        """Download the song lyrics from the API and translate them using the Lyrics class."""
        lyrics = Lyrics(song=song, artist=artist, testing=testing)
        lyrics.download_lyrics(genius=self.genius)
        lyrics.translate(translator=self.translator, language=self.language)
        return lyrics

    def __repr__(self) -> str:
        """LyricsTranslator representation

        Returns:
            str: returns the string to create the same instance
        """
        return (
            f"LyricsTranslator("
            f" language={self.language}, origin_language={self.origin_language})"
        )



import textwrap

import click

from lyrics_translator import LyricsTranslator

from . import __version__


@click.command()
@click.option(
    "--song", default="Surfin' U.S.A.", help="Name of the song to be translated."
)
@click.option("--artist", default="The Beach Boys", help="Name of the artist.")
@click.option(
    "--language", default="de", help="Language to song lyrics to translate to."
)
@click.option(
    "--testing",
    default=False,
    help=('Mode of developement, if testing is set to "True" only dummy data is used'),
)
@click.version_option(version=__version__)
def main(song, artist, language, testing):

    translator = LyricsTranslator(language)
    lyrics = translator.get_song_translation(song, artist, testing)
    lyrics.get_song_translation()

    title = f"'{song}' by '{artist}' translated into '{language}'"

    click.secho(title, fg="green")
    click.echo(lyrics)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lyrics_translator', 'lyrics_translator.core']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'lyricsgenius>=3.0.1,<4.0.0',
 'python-docx>=0.8.11,<0.9.0',
 'python-dotenv>=0.20.0,<0.21.0',
 'sacremoses>=0.0.53,<0.0.54',
 'sentencepiece>=0.1.96,<0.2.0',
 'tqdm>=4.64.1,<5.0.0',
 'transformers[torch]>=4.20.1,<5.0.0']

entry_points = \
{'console_scripts': ['lyrics-translator = lyrics_translator.console:main']}

setup_kwargs = {
    'name': 'lyrics-translator',
    'version': '0.3.0',
    'description': '🎵 LyricsTranslator - automated lyrics translation',
    'long_description': '\n<!-- <p align="center">\n<img src="https://github.com/MauroLuzzatto/lyrics-translator/blob/main/docs/img/logo.jpg" width="200" height="200"/>\n</p> -->\n\n<h1 align="center">🎵 LyricsTranslator - automated lyrics translation</h1>\n\n\n\n<p align="center">\n\n<a href="https://pypi.python.org/pypi/lyrics-translator" target="_blank">\n    <img src="https://img.shields.io/pypi/v/lyrics-translator.svg" alt="pypi version">\n</a>\n\n<a href="https://pypi.org/project/lyrics-translator" target="_blank">\n    <img src="https://img.shields.io/pypi/pyversions/lyrics-translator.svg" alt="Supported versions">\n</a>\n\n<a href="https://github.com/ambv/black" target="_blank">\n    <img src="https://img.shields.io/badge/code%20style-black-000000.svg?style=flat-square" alt="Code style: black">\n\n<a href="https://pycqa.github.io/isort/" target="_blank">\n    <img src="https://img.shields.io/badge/%20imports-isort-%231674b1?style=flat&labelColor=ef8336" alt="Imports: isort">\n\n\n</p>\n\n---\n\n**Documentation**: <a href="https://mauroluzzatto.github.io/lyrics-translator" target="_blank">https://mauroluzzatto.github.io/lyrics-translator</a>\n\n**Source Code**: <a href="https://github.com/MauroLuzzatto/lyrics-translator" target="_blank">https://github.com/MauroLuzzatto/lyrics-translator</a>\n\n---\n\n\nThe `LyricsTranslator` downloads lyrics from [genius](https://genius.com/) and uses 🤗[hugging face](https://huggingface.co/) to translate the lyrics into a target language.\n\n\nAll languages that are supported by [OPUS-MT](https://github.com/Helsinki-NLP/Opus-MT) are available for translation.The full list of list of languages can be found on 🤗[hugging face](https://huggingface.co/models?other=marian).\n\n- German: `de`\n- Swedish: `sv`\n- French: `fr`\n- Spanish: `es` \n- Chinese: `zh`\n- Japanese: `ja`\n- Portuguese: `pt`\n- Arabic: `ar`\n- Italian: `it`\n\nand many more ...\n\n\n\n\n\n## Install\n\n```\npip install lyrics-translator\n```\n\n## Setup\nTo use the `LyricsTranslator` you will have to [get an API token](https://docs.genius.com/#/getting-started-h1) from `genius` add the API token to the `.env` file:\n\n```txt\nGENIUS_ACCESS_TOKEN=<replace-me-with-your-genius-api-token>\n```\n\n## Supported Languages\n\n\n\n## Usage\n<!--\n📚 A comprehensive example of the `explainy` API can be found in this ![Jupyter Notebook](https://github.com/MauroLuzzatto/explainy/blob/main/examples/01-explainy-intro.ipynb)\n\n📖 Or in the [example section](https://explainy.readthedocs.io/en/latest/examples/01-explainy-intro.html) of the documentation -->\n\n\n```python\nfrom lyrics_translator import LyricsTranslator\n\nsong = "Surfin\' U.S.A."\nartist = "The Beach Boys"\nlanguage = "de"\n\ntranslator = LyricsTranslator(language)\nlyrics = translator.get_song_translation(song, artist)\nprint(lyrics)\n```\nOutput:\n```\nSurfin’ USA Lyrics[Verse 1]\nWenn jeder einen Ozean hätte\nÜberall in den USA\nDann würde jeder surfen\nWie Californi-a\nSie würden ihre Taschen tragen.\nAuch Huarache Sandalen\nEin stumpfes stumpfes blond Haar\nSurfin\' U.S.A\n\n[Korus]\nSie würden sie surfen in Del Mar\n(Innen, Außen, USA)\nVentura County Line\n(Innen, Außen, USA)\nSanta Cruz und Trestles\n(Innen, Außen, USA)\nAustraliens Narrabeen\n(Innen, Außen, USA)\nÜberall in Manhattan\n(Innen, Außen, USA)\nUnd den Doheny Way hinunter\n(Innen, Außen)\n[Anschlag]\nJeder ist surfin\'\nSurfin\' U.S.A\n\n[Zwischenruf 2]\nWir werden alle diese Route planen.\nWir werden wirklich bald\nWir wischen unsere Surfbretter ab\nWir können auf Juni nicht warten\nWir werden alle für den Sommer weg sein\nWir sind auf surfari zu bleiben\nSagen Sie dem Lehrer, wir surfen\nSurfin\' U.S.A\n\n[Korus]\nHaggerties und Swamis\n(Innen, Außen, USA)\nPalisaden im Pazifik\n(Innen, Außen, USA)\nSan Onofre und der Sonnenuntergang\n(Innen, Außen, USA)\nRedondo Beach LA\n(Innen, Außen, USA)\nGanz La Jolla\n(Innen, Außen, USA)\nIn der Bucht von Wa\'imea\n(Innen, Außen)\n[Anschlag]\nJeder ist surfin\'\nSurfin\' U.S.A\n\n[Instrumental Interlude]\n\n[Outro]\nJeder ist surfin\'\nSurfin\' U.S.A\n\nJeder ist surfin\'\nSurfin\' U.S.A\n\nJeder ist surfin\'\nSurfin\' U.S.A\n\nJeder ist surfin\'\nSurfin\' U.S.A\n```\n',
    'author': 'Mauro Luzzatto',
    'author_email': 'mauroluzzatto@hotmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://mauroluzzatto.github.io/lyrics-translator',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
